@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 # @Author : 陈坤泽
 # @Email  : 877362867@qq.com
-# @Data   : 2020/05/30 20:37
+# @Date   : 2020/05/30 20:37
 
 
 from urllib.parse import urlparse
@@ -99,7 +99,7 @@ def test_etag2():
     print(get_etag(s))
     # FkAD2McB6ugxTiniE8ebhlNHdHh9
 
-    f = File('1.tex', root=File.TEMP).write(s, if_exists='replace').fullpath
+    f = File('1.tex', tempfile.gettempdir()).write(s, if_exists='delete').to_str()
     print(get_etag(f))
     # FkAD2McB6ugxTiniE8ebhlNHdHh9
 
@@ -108,18 +108,31 @@ ____chardet = """
 """
 
 
-def get_encoding(bstr, maxn=1024):
+def get_encoding(bstr):
     """ 输入二进制字符串或文本文件，返回字符编码
 
     https://www.yuque.com/xlpr/pyxllib/get_encoding
 
     :param bstr: 二进制字符串、文本文件
-    :param maxn: 分析字节数上限，越小速度越快，但也会降低精准度
     :return: utf8, utf-8-sig, gbk, utf16
     """
+
+    # 0 检查工具
+    def inner_detect(bdata):
+        """ https://mp.weixin.qq.com/s/gSXNf3K_JWydhej8KpyhMg """
+        from chardet.universaldetector import UniversalDetector
+        detector = UniversalDetector()
+        for line in bdata.splitlines():
+            detector.feed(line)
+            if detector.done:
+                break
+        detector.close()
+        return detector.result['encoding']
+
     # 1 读取编码
     if isinstance(bstr, bytes):  # 如果输入是一个二进制字符串流则直接识别
-        encoding = chardet.detect(bstr[:maxn])['encoding']  # 截断一下，不然太长了，太影响速度
+        # encoding = chardet.detect(bstr[:maxn])['encoding']  # 截断一下，不然太长了，太影响速度
+        encoding = inner_detect(bstr)  # 截断一下，不然太长了，太影响速度
     elif is_file(bstr):  # 如果是文件，则按二进制打开
         # 如果输入是一个文件名则进行读取
         if bstr.endswith('.pdf'):
@@ -127,7 +140,8 @@ def get_encoding(bstr, maxn=1024):
             return 'utf8'
         with open(bstr, 'rb') as f:  # 以二进制读取文件，注意二进制没有\r\n的值
             bstr = f.read()
-        encoding = chardet.detect(bstr[:maxn])['encoding']
+        # encoding = chardet.detect(bstr[:maxn])['encoding']
+        encoding = inner_detect(bstr)
     else:  # 其他类型不支持
         return 'utf8'
     # 检测结果存储在encoding
@@ -144,7 +158,7 @@ def get_encoding(bstr, maxn=1024):
         # 保留原值的一些正常识别结果
         encoding = 'utf-8-sig'
     elif bstr.strip():  # 如果不在预期结果内，且bstr非空，则用常见的几种编码尝试
-        # dprint(encoding)
+        # print('encoding=', encoding)
         type_ = ('utf8', 'gbk', 'utf16')
 
         def try_encoding(bstr, encoding):
@@ -164,188 +178,116 @@ def get_encoding(bstr, maxn=1024):
 
 
 ____file = """
+路径、文件、目录相关操作功能
+
+主要是为了提供readfile、wrritefile函数
+与普通的读写文件相比，有以下优点：
+1、智能识别pkl等特殊格式文件的处理
+2、智能处理编码
+3、目录不存在自动创建
+4、自动备份旧文件，而不是强制覆盖写入
+
+其他相关文件处理组件：isfile、get_encoding、ensure_folders
 """
 
 
-class File:
-    r""" 通用文件、路径处理类，也可以处理目录，可以把目录对象理解为特殊类型的文件
-        其实叫File比较好，因为主要就是针对文件处理，而不是路径解析
-        如果是纯粹的路径解析，推荐pathlib.Path处理
+class PathBase:
+    """ File和Dir共有的操作逻辑功能 """
+    __slots__ = ('_path',)
 
-    document: https://www.yuque.com/xlpr/python/pyxllib.debug.path
-
-    大部分基础功能是从pathlib.Path衍生过来的
-        但开发中该类不能直接从Path继承，会有很多问题
-
-    TODO 这里的doctest过于针对自己的电脑了，应该改成更具适用性测试代码
-    """
-    __slots__ = ('_path', 'assume_dir')
-
-    # 零、常用的目录类
-    TEMP = tempfile.gettempdir()
-    if os.environ.get('Desktop', None):  # 如果修改了win10默认的桌面路径，需要在环境变量添加一个正确的Desktop路径值
-        DESKTOP = os.environ['Desktop']
-    else:
-        DESKTOP = os.path.join(str(pathlib.Path.home()), 'Desktop')  # 这个不一定准，桌面是有可能被移到D盘等的
-
-    # 一、基础功能
-
-    def __init__(self, path=None, suffix=None, root=None):
-        r""" 初始化参数含义详见 abspath 函数解释
-
-        TODO 这个初始化也有点过于灵活了，需要降低灵活性，增加使用清晰度
-            或者看下性能速度，可以考虑加速
-            特别是用Path作为参数拷贝的情况，应该可以加速减少分析、拷贝
-
-        >>> File('D:/pycode/code4101py')
-        File('D:/pycode/code4101py')
-        >>> File(File('D:/pycode/code4101py'))
-        File('D:/pycode/code4101py')
-
-        >> File()  # 不输入参数的时候，默认为当前工作目录
-        File('D:/pycode/code4101py')
-
-        >> File('a.txt', root=Path.TEMP)
-        File('D:/Temp/a.txt')
-        >>> File('F:/work/CreatorTemp')
-        File('F:/work/CreatorTemp')
-
-        注意！如果使用了符号链接（软链接），则路径是会解析转向实际位置的！例如
-        >> File('D:/pycode/code4101py')
-        File('D:/slns/pycode/code4101py')
-        """
-        strpath = str(path)
-        self._path = None
-        self.assume_dir = False  # 假设是一个目录
-        if len(strpath) and strpath[-1] in r'\/':
-            self.assume_dir = True
-
-        try:
-            self._path = pathlib.Path(self.abspath(path, suffix, root)).resolve()
-            # 有些问题上一步不一定测的出来，要再补一个测试
-            # self._path.is_file()
-        except (ValueError, TypeError, OSError):
-            # ValueError：文件名过长，代表输入很可能是一段文本，根本不是路径
-            # TypeError：不是str等正常的参数
-            # OSError：非法路径名，例如有 *? 等
-            self._path = None
-
-    @staticmethod
-    def abspath(path=None, suffix=None, root=None) -> str:
+    @classmethod
+    def abspath(cls, path=None, root=None, *, suffix=None) -> pathlib.Path:
         r""" 根据各种不同的组合参数信息，推导出具体的路径位置
 
-        (即把人常识易理解的指定思维，转成计算机能清晰明白的具体指向对象)
         :param path: 主要的参数，如果后面的参数有矛盾，以path为最高参考标准
-            ''，可以输入空字符串，效果跟None是不一样的，意思是显示地指明要生成一个随机名称的文件名
+            ...，可以输入Ellipsis对象，效果跟None是不一样的，意思是显式地指明要生成一个随机名称的文件
+            三个参数全空时，则返回当前目录
         :param suffix:
             以 '.' 指明的扩展名，会强制替换
             否则，只作为参考扩展名，只在原有path没有指明的时候才添加
         :param root: 未输入的时候，则为当前工作目录
 
-        >> Path.abspath()
-        'C:\\pycode\\code4101py'
+        >>> os.chdir("C:/Users")
 
-        >> Path.abspath('a')
-        'C:\\pycode\\code4101py\\a'
+        # 未输入任何参数，则返回当前工作目录
+        >>> PathBase.abspath()
+        WindowsPath('C:/Users')
 
-        >> Path.abspath('F:/work', '.txt')  # F:/work是个实际存在的目录
-        'F:/work\\tmpahqm6nod.txt'
+        # 基本的指定功能
+        >>> PathBase.abspath('a')
+        WindowsPath('C:/Users/a')
 
-        >> Path.abspath('F:/work/a.txt', 'py')  # 参考后缀不修改
-        'F:/work/a.txt'
-        >> Path.abspath('F:/work/a.txt', '.py')  # 强制后缀会修改
-        'F:/work/a.py'
+        # 额外指定父目录
+        >>> PathBase.abspath('a/b', 'D:')
+        WindowsPath('D:/a/b')
 
-        >> Path.abspath(suffix='.tex', root=Path.TEMP)
-        'F:\\work\\CreatorTemp\\tmp5vo2lpqd.tex'
+        # 设置扩展名
+        >>> PathBase.abspath('a', suffix='.txt')
+        WindowsPath('C:/Users/a.txt')
 
-        # F:/work/a.txt不存在，而且看起来像文件名，但是末尾再用/则显式指明这其实是一个目录
-        #   又用.py指明要添加一个随机名称的py文件
-        >> Path.abspath('F:/work/a.txt/', '.py')
-        'F:/work/a.txt/tmpg_q7a7ft.py'
+        # 扩展名的高级用法
+        >>> PathBase.abspath('F:/work/a.txt', suffix='py')  # 参考后缀不修改
+        WindowsPath('F:/work/a.txt')
+        >>> PathBase.abspath('F:/work/a.txt', suffix='.py')  # 强制后缀会修改
+        WindowsPath('F:/work/a.py')
+        >>> PathBase.abspath('F:/work/a.txt', suffix='')  # 删除后缀
+        WindowsPath('F:/work/a')
 
-        >> Path.abspath('work/a.txt/', '.py', Path.TEMP)  # 在临时文件夹下的work/a.txt目录新建一个随机名称的py文件
-        'F:\\work\\CreatorTemp\\work/a.txt/tmp2jn5cqkc.py'
-
-        >> Path.abspath('C:/a.txt/')  # 会保留最后的斜杠特殊标记
-        'C:/a.txt/'
-        >> Path.abspath('C:/a.txt\\')  # 会保留最后的斜杠特殊标记
-        'C:/a.txt\\'
+        # 在临时目录下，新建一个.tex的随机名称文件
+        >> PathBase.abspath(..., tempfile.gettempdir(), suffix='.tex')
+        WindowsPath('D:/Temp/tmp_sey0yeg.tex')
         """
         # 1 判断参考目录
-        if root:
-            root = str(root)
-        else:
+        if root is None:
             root = os.getcwd()
+        else:
+            root = str(root)
 
         # 2 判断主体文件名 path
-        if str(path) == '':
+        if path is None:
+            return pathlib.Path(root)
+        elif path is Ellipsis:
             path = tempfile.mktemp(dir=root)
-        elif not path:
-            path = root
         else:
             path = os.path.join(root, str(path))
 
         # 3 补充suffix
-        if suffix:
-            # 如果原来的path只是一个目录，则要新建一个文件
-            # if os.path.isdir(path) or path[-1] in ('\\', '/'):
-            if path[-1] in ('\\', '/'):
-                path = tempfile.mktemp(dir=path)
+        if suffix is not None:
             # 判断后缀
             li = os.path.splitext(path)
-            if (not li[1]) or suffix[0] == '.':
-                ext = suffix if suffix[0] == '.' else ('.' + suffix)
-                path = li[0] + ext
+            if suffix == '' or suffix[0] == '.':
+                path = li[0] + suffix
+            elif li[1] == '':
+                path = li[0] + '.' + suffix
 
-        return path
+        return pathlib.Path(path).resolve()
 
     def __bool__(self):
-        return bool(self._path)
-
-    def exists(self):
-        r""" 判断文件是否存在
+        r""" 判断文件、文件夹是否存在
 
         重置WindowsPath的bool逻辑，返回值变成存在True，不存在为False
 
-        >>> File('D:/slns').exists()
+        >>> bool(File('C:/Windows/System32/cmd.exe'))
         True
-        >>> File('D:/pycode/code4101').exists()
+        >>> bool(File('C:/Windows/System32/cmdcmd.exe'))
         False
         """
-        return self._path and self._path.exists()
+        return self._path.exists()
 
     def __repr__(self):
         s = self._path.__repr__()
         if s.startswith('WindowsPath'):
-            s = 'File' + s[11:]
+            s = self.__class__.__name__ + s[11:]
         elif s.startswith('PosixPath'):
-            s = 'File' + s[9:]
+            s = self.__class__.__name__ + s[9:]
         return s
 
     def __str__(self):
-        return str(self._path).replace('\\', '/') + ('/' if self.assume_dir else '')
+        return str(self._path).replace('\\', '/')
 
     def to_str(self):
-        """ 不推荐使用，建议直接用str """
         return self.__str__()
 
-    def __eq__(self, other):
-        """ pathlib.Path内置了windows和linux的区别
-            在windows是不区分大小写的，在linux则区分大小写
-        """
-        if not isinstance(other, File):
-            raise TypeError
-        return self._path == other._path
-
-    def __truediv__(self, key):
-        r""" 路径拼接功能
-
-        >>> File('C:/a') / 'b.txt'
-        File('C:/a/b.txt')
-        """
-        return File(self._path / str(key))
-
     def resolve(self):
         return self._path.resolve()
 
@@ -355,12 +297,6 @@ class File:
     def match(self, path_pattern):
         return self._path.match(path_pattern)
 
-    # 二、获取、修改路径中部分值的功能
-
-    @property
-    def fullpath(self) -> str:
-        return str(self._path)
-
     @property
     def drive(self) -> str:
         return self._path.drive
@@ -390,43 +326,20 @@ class File:
     def parent(self):
         r"""
         >>> File('D:/pycode/code4101py').parent
-        File('D:/pycode')
+        WindowsPath('D:/pycode')
         """
-        return File(self._path.parent) if self._path else None
+        return self._path.parent
 
     @property
     def dirname(self) -> str:
         r"""
         >>> File('D:/pycode/code4101py').dirname
-        'D:/pycode'
+        'D:\\pycode'
         >>> File(r'D:\toweb\a').dirname
-        'D:/toweb'
+        'D:\\toweb'
         """
         return str(self.parent)
 
-    def with_dirname(self, value):
-        return File(self.name, root=value)
-
-    @property
-    def stem(self) -> str:
-        r"""
-        >>> File('D:/pycode/code4101py/ckz.py').stem
-        'ckz'
-        >>> File('D:/pycode/.gitignore').stem  # os.path.splitext也是这种算法
-        '.gitignore'
-        >>> File('D:/pycode/.123.45.6').stem
-        '.123.45'
-        """
-        return self._path.stem
-
-    def with_stem(self, stem):
-        """
-        注意不能用@stem.setter来做
-            如果setter要rename，那if_exists参数怎么控制？
-            如果setter不要rename，那和用with_stem实现有什么区别？
-        """
-        return File(stem, self.suffix, self.dirname)
-
     @property
     def parts(self) -> tuple:
         r"""
@@ -435,45 +348,6 @@ class File:
         """
         return self._path.parts
 
-    @property
-    def suffix(self) -> str:
-        r"""
-        >>> File('D:/pycode/code4101py/ckz.py').suffix
-        '.py'
-        >>> File('D:/pycode/code4101py').suffix
-        ''
-        >>> File('D:/pycode/code4101py/ckz.').suffix
-        ''
-        >>> File('D:/pycode/code4101py/ckz.123.456').suffix
-        '.456'
-        >>> File('D:/pycode/code4101py/ckz.123..456').suffix
-        '.456'
-        >>> File('D:/pycode/.gitignore').suffix
-        ''
-        """
-        return self._path.suffix if self._path else ''
-
-    def with_suffix(self, suffix):
-        r""" 指向同目录下后缀为suffix的文件
-
-        >>> File('a.txt').with_suffix('.py').fullpath.split('\\')[-1]  # 强制替换
-        'a.py'
-        >>> File('a.txt').with_suffix('py').fullpath.split('\\')[-1]  # 参考替换
-        'a.txt'
-        >>> File('a.txt').with_suffix('').fullpath.split('\\')[-1]  # 删除
-        'a'
-        """
-        if suffix and (suffix[0] == '.' or not self.suffix):
-            if suffix[0] != '.': suffix = '.' + suffix
-            return File(self._path.with_suffix(suffix))
-        elif not suffix:
-            # suffix 为假值则删除扩展名
-            return File(self.stem, '', self.dirname)
-        return self
-
-    def joinpath(self, *args):
-        return File(self._path.joinpath(*args))
-
     @property
     def backup_time(self):
         r""" 返回文件的备份时间戳，如果并不是备份文件，则返回空字符串
@@ -496,46 +370,6 @@ class File:
         g = re.match(r'(\d{6}-\d{6})', name[-13:])
         return g.group(1) if g else ''
 
-    # 三、获取文件相关属性值功能
-
-    def is_dir(self):
-        return self._path and self._path.is_dir()
-
-    def is_file(self):
-        return self._path and self._path.is_file()
-
-    @property
-    def encoding(self):
-        """ 文件的编码
-
-        非文件、不存在时返回 None
-        """
-        if self.is_file():
-            return get_encoding(self.fullpath)
-        return None
-
-    @property
-    def size(self) -> int:
-        """ 计算文件、目录的大小，对于目录，会递归目录计算总大小
-
-        https://stackoverflow.com/questions/1392413/calculating-a-directory-size-using-python
-
-        >> Path('D:/slns/pyxllib').size  # 这个算的就是真实大小，不是占用空间
-        2939384
-        """
-        path = str(self._path)
-        if self._path.is_file():
-            total_size = os.path.getsize(path)
-        elif self._path.is_dir():
-            total_size = 0
-            for dirpath, dirnames, Pathnames in os.walk(path):
-                for f in Pathnames:
-                    fp = os.path.join(dirpath, f)
-                    total_size += os.path.getsize(fp)
-        else:  # 不存在的对象
-            total_size = 0
-        return total_size
-
     @property
     def mtime(self) -> Datetime:
         r""" 文件的最近修改时间
@@ -543,7 +377,7 @@ class File:
         >> Path(r"C:\pycode\code4101py").mtime
         2020-03-10 17:32:37
         """
-        return Datetime(os.stat(self.fullpath).st_mtime)
+        return Datetime(os.stat(str(self)).st_mtime)
 
     @property
     def ctime(self) -> Datetime:
@@ -553,197 +387,78 @@ class File:
         2018-05-25 10:46:37
         """
         # 注意：st_ctime是平台相关的值，在windows是创建时间，但在Unix是metadate最近修改时间
-        return Datetime(os.stat(self.fullpath).st_ctime)
+        return Datetime(os.stat(str(self)).st_ctime)
 
-    # 四、文件操作功能
-
-    def abs_dstpath(self, dst=None, suffix=None, root=None):
-        r""" 参照当前Path的父目录，来确定dst的具体路径
-
-        >>> f = File('C:/Windows/System32/cmd.exe')
-        >>> f.abs_dstpath('chen.py')
-        File('C:/Windows/System32/chen.py')
-        >>> f.abs_dstpath('E:/')  # 原始文件必须存在，否则因为无法判断实际类型，目标路径可能会错
-        File('E:/cmd.exe')
-        >>> f.abs_dstpath('D:/aabbccdd.txt')
-        File('D:/aabbccdd.txt')
-        >>> f.abs_dstpath('D:/aabbccdd.txt/')  # 并不存在aabbccdd.txt这样的对象，但末尾有个/表明这是个目录
-        File('D:/aabbccdd.txt/cmd.exe')
-        """
-        if not root: root = self.dirname
-        dst = File(dst, suffix, root)
-        # print(dst, dst.assume_dir)
-
-        if self.is_dir() or (self.assume_dir and not self.is_file()):
-            if dst.is_file():
-                raise ValueError(f'{dst}是已存在的文件类型，不能对{self}目录执行指定操作')
-            elif dst.assume_dir:  # 明确要把目录放到另一个目录下
-                dst = dst / self.name
-        else:  # 否则self是文件，或者不存在，均视为文件处理
-            if dst.is_dir() or dst.assume_dir:
-                dst = dst / self.name
-            # 否则dst是文件，或者不存在的路径，均视为文件类型处理
-
-        return dst
-
-    def relative_path(self, ref_dir) -> str:
+    def relpath(self, ref_dir) -> str:
         r""" 当前路径，相对于ref_dir的路径位置
 
-        >>> File('C:/a/b/c.txt').relative_path('C:/a/')
+        >>> File('C:/a/b/c.txt').relpath('C:/a/')
         'b\\c.txt'
-        >>> File('C:/a/b\\c.txt').relative_path('C:\\a/')
+        >>> File('C:/a/b\\c.txt').relpath('C:\\a/')
         'b\\c.txt'
+
+        >> File('C:/a/b/c.txt').relpath('D:/')  # ValueError
         """
-        if not isinstance(ref_dir, File):
-            ref_dir = File(ref_dir)
-        # s1, s2 = str(self), str(ref_dir)
-        # if s1.startswith(s2):
-        #     s1 = s1[len(s2):]
         return os.path.relpath(str(self), str(ref_dir))
 
-    def preprocess(self, if_exists='error', exclude=None):
-        """ 这个功能要结合参数一起理解，不是简单的理解成“预处理”
-        这个实际上是在做copy等操作前，如果目标文件已存在，需要预先删除等的预处理
+    def exist_preprcs(self, if_exists=None):
+        """ 这个实际上是在做copy等操作前，如果目标文件已存在，需要预先删除等的预处理
         并返回判断，是否需要执行下一步操作
 
         有时候情况比较复杂，process无法满足需求时，可以用preprocess这个底层函数协助
 
         :param if_exists:
-            'error': （默认）如果要替换的目标文件已经存在，则报错
-            'overwrite', 'replace': 替换 （提前把已存在的目标文件删除）
-            'skip', 'ignore': 忽略、不处理  （不用执行后续的功能）
-            'backup': 备份后写入  （对原文件先做一个备份）
-        :param exclude: 排除掉不分析的目录，用于有些重命名等自身可能操作自身的情况
-            如果self是exclude这个路径，默认直接need_run=True
+            None: 不做任何处理，直接运行，依赖于功能本身是否有覆盖写入机制
+            'error': 如果要替换的目标文件已经存在，则报错
+            'delete': 把存在的文件先删除
+            'skip': 不执行后续功能
+            'backup': 先做备份  （对原文件先做一个备份）
         """
-        # 1 如果src和dst是同一个文件，因为重命名等特殊功能，可以直接执行，不用管提前存在目标文件的问题
-        if exclude and self == File(exclude):
-            return True
-
-        # 2
         need_run = True
-        if self.exists():
-            if if_exists == 'error':
+        if self:
+            if if_exists is None:
+                return need_run
+            elif if_exists == 'error':
                 raise FileExistsError(f'目标文件已存在： {self}')
-            elif if_exists in ('replace', 'overwrite'):  # None的话相当于replace，但是不会事先delete，可能会报错
+            elif if_exists == 'delete':
                 self.delete()
-            elif if_exists in ('ignore', 'skip'):
+            elif if_exists == 'skip':
                 need_run = False
             elif if_exists == 'backup':
-                self.backup(if_exists='backup')
-                self.delete()
+                self.backup(move=True)
+            else:
+                raise ValueError(f'{if_exists}')
         return need_run
 
-    def process(self, dst, func, if_exists='error'):
-        r""" copy或move的本质底层实现
-
-        文件复制等操作中src、dst不同组合下的效果：https://www.yuque.com/xlpr/pyxllib/mgwe19
-
-        :param dst: 目标路径对象，注意如果使用相对路径，是相对于self的路径！
-        :param func: 传入arg1和arg2参数，可以自定义
-            默认分别是self和dst的fullpath
-        :return : 返回dst
-        """
-        # 1 判断目标是有已存在，进行不同的指定规则处理
-        dst0 = self.abs_dstpath(dst)
-
-        # 2 执行特定功能
-        if dst0.preprocess(if_exists, self):
-            # 此时dst已是具体路径，哪怕是"目录"也可以按照"文件"对象理解，避免目录会重复生成，多层嵌套
-            #   本来是 a --> C:/target/a ，避免 C:/target/a/a 的bug
-            dst0.ensure_dir('file')
-
-            if dst0 == self:  # 重命名自身的操作比较特别，需要打补丁
-                func(self.fullpath, os.path.join(dst0.dirname, pathlib.Path(str(dst)).name))
-                dst0 = self.abs_dstpath(dst)
-            else:
-                func(self.fullpath, dst0.fullpath)
-
-        return dst0
-
-    def ensure_dir(self, pathtype=None):
-        r""" 确保path中指定的dir都存在
-
-        :param pathtype: 如果self.path的对象不存在，根据self.path的类型不同，有不同的处理方案
-            'dir'：则包括自身也会创建一个空目录
-            'file'： 只会创建到dirname所在目录，并不会创建自身的Path
-            'None'：通过名称智能判断，如果能读取到suffix，则代表是Path类型，否则是dir类型
-        :return:
-
-        >> Path(r'D:\toweb\a\b.txt').ensure_dir()  # 只会创建toweb、a目录
-
-        # a和一个叫b.txt的目录都会创建
-        # 当然，如果b.txt是一个已经存在的文件对象，则该函数不会进行操作
-        >> Path(r'D:\toweb\a\b.txt').ensure_dir(pathtype='dir')
-        """
-        if not self.exists():
-            if not pathtype:
-                pathtype = 'file' if (not self.assume_dir and self.suffix) else 'dir'
-            dirname = self.fullpath if pathtype == 'dir' else self.dirname
-            if not os.path.exists(dirname):
-                os.makedirs(dirname)
-
-    def copy(self, dst, if_exists='error'):
-        """ 复制文件
-
-        """
-        if self.is_dir():
-            return self.process(dst, shutil.copytree, if_exists)
-        elif self.is_file():
-            return self.process(dst, shutil.copy2, if_exists)
-
-    def move(self, dst, if_exists='error'):
-        """ 移动文件
-
-        """
-        if self.exists():
-            return self.process(dst, shutil.move, if_exists)
-
-    def rename(self, dst, if_exists='error'):
-        r""" 文件重命名，或者也可以理解成文件移动
-
-        :param dst: 如果使用相对目录，则是该类本身所在的目录作为工作环境
-        :param if_exists:
-            'error': 如果要替换的目标文件已经存在，则报错
-            'replace': 替换
-            'ignore': 忽略、不处理
-            'backup': 备份后替换
-        :return:
-        """
-        # rename是move的一种特殊情况
-        return self.move(dst, if_exists)
+    def copy(self, *args, **kwargs):
+        raise NotImplementedError
 
     def delete(self):
-        r""" 删除自身文件
+        raise NotImplementedError
 
-        """
-        if self.is_file():
-            os.remove(self.fullpath)
-        elif self.is_dir():
-            shutil.rmtree(self.fullpath)
-        # TODO 确保删除后再执行后续代码 但是一直觉得这样写很别扭
-        while self.exists(): pass
+    def absdst(self, dst):
+        raise NotImplementedError
 
-    def backup(self, tail=None, if_exists='replace', move=False):
+    def backup(self, tail=None, if_exists='delete', move=False):
         r""" 对文件末尾添加时间戳备份，也可以使用自定义标记tail
 
         :param tail: 自定义添加后缀
             tail为None时，默认添加特定格式的时间戳
         :param if_exists: 备份的目标文件名存在时的处理方案
-        :param move:
-            是否删除原始文件
+            这个概率非常小，真遇到，先把已存在的删掉，重新写入一个是可以接受的
+        :param move: 是否删除原始文件
 
         # TODO：有个小bug，如果在不同时间实际都是相同一个文件，也会被不断反复备份
         #    如果想解决这个，就要读取目录下最近的备份文件对比内容了
         """
         # 1 判断自身文件是否存在
-        if not self.exists():
+        if not self:
             return None
 
         # 2 计算出新名称
         if not tail:
             tail = self.mtime.strftime(' %y%m%d-%H%M%S')  # 时间戳
-        name, ext = os.path.splitext(self.fullpath)
+        name, ext = os.path.splitext(str(self))
         dst = name + tail + ext
 
         # 3 备份就是特殊的copy操作
@@ -752,6 +467,202 @@ class File:
         else:
             return self.copy(dst, if_exists)
 
+    def move(self, dst, if_exists=None):
+        """ 移动文件
+        """
+        if self:
+            return self.process(dst, shutil.move, if_exists)
+
+    def process(self, dst, func, if_exists=None):
+        r""" copy或move的本质底层实现
+
+        :param dst: 目标路径对象
+        :param func: 传入arg1和arg2参数，可以自定义
+            默认分别是self和dst的字符串
+        :return : 返回dst
+        """
+        # 1 判断目标是有已存在，进行不同的指定规则处理
+        dst_ = self.absdst(dst)
+
+        # 2 执行特定功能
+        if self == dst_:
+            # 如果文件是自身的话，并不算exists，可以直接run，不用exist_preprcs
+            dst_.ensure_parent()
+            func(str(self), str(dst_))
+            dst_._path = dst_._path.resolve()  # 但是需要重新解析一次dst_._path，避免可能有重命名等大小写变化
+        elif dst_.exist_preprcs(if_exists):
+            dst_.ensure_parent()
+            func(str(self), str(dst_))
+
+        return dst_
+
+    def explorer(self, proc='explorer'):
+        """ 使用windows的explorer命令打开文件
+
+        还有个类似的万能打开命令 start
+
+        :param proc: 可以自定义要执行的主程序
+        """
+        subprocess.run([proc, str(self)])
+
+    def ensure_parent(self):
+        r""" 确保父目录存在
+        """
+        p = self.parent
+        if not p.exists():
+            os.makedirs(str(p))
+
+
+class File(PathBase):
+    r""" 通用文件处理类，大部分基础功能是从pathlib.Path衍生过来的
+
+    document: https://www.yuque.com/xlpr/python/pyxllib.debug.path
+    """
+
+    # 一、基础功能
+
+    def __init__(self, path, root=None, *, suffix=None):
+        r""" 初始化参数含义详见 PathBase.abspath 函数解释
+
+        注意！如果使用了符号链接（软链接），则路径是会解析转向实际位置的！例如
+        >> File('D:/pycode/code4101py')
+        File('D:/slns/pycode/code4101py')
+        """
+        self._path = None
+        # 1 快速初始化
+        if root is None and suffix is None:
+            if isinstance(path, File):
+                self._path = path._path
+                return  # 直接完成初始化过程
+            elif isinstance(path, pathlib.Path):
+                self._path = path
+        # 2 普通初始化
+        if self._path is None:
+            self._path = self.abspath(path, root, suffix=suffix)
+        if not self._path:
+            raise ValueError(f'无效路径 {self._path}')
+        elif self._path.is_dir():
+            raise ValueError(f'不能用目录初始化一个File对象 {self._path}')
+
+    @classmethod
+    def safe_init(cls, path, root=None, *, suffix=None):
+        """ 如果失败不raise，而是返回None的初始化方式 """
+        try:
+            f = File(path, root, suffix=suffix)
+            f._path.is_file()  # 有些问题上一步不一定测的出来，要再补一个测试
+            return f
+        except (ValueError, TypeError, OSError, PermissionError):
+            # ValueError：文件名过长，代表输入很可能是一段文本，根本不是路径
+            # TypeError：不是str等正常的参数
+            # OSError：非法路径名，例如有 *? 等
+            # PermissionError: linux上访问无权限、不存在的路径
+            return None
+
+    # 二、获取、修改路径中部分值的功能
+
+    def with_dirname(self, value):
+        return File(self.name, value)
+
+    @property
+    def stem(self) -> str:
+        r"""
+        >>> File('D:/pycode/code4101py/ckz.py').stem
+        'ckz'
+        >>> File('D:/pycode/.gitignore').stem  # os.path.splitext也是这种算法
+        '.gitignore'
+        >>> File('D:/pycode/.123.45.6').stem
+        '.123.45'
+        """
+        return self._path.stem
+
+    def with_stem(self, stem):
+        """
+        注意不能用@stem.setter来做
+            如果setter要rename，那if_exists参数怎么控制？
+            如果setter不要rename，那和用with_stem实现有什么区别？
+        """
+        return File(stem, self.parent, suffix=self.suffix)
+
+    @property
+    def suffix(self) -> str:
+        r"""
+        >>> File('D:/pycode/code4101py/ckz.py').suffix
+        '.py'
+        >>> File('D:/pycode/code4101py').suffix
+        ''
+        >>> File('D:/pycode/code4101py/ckz.').suffix
+        ''
+        >>> File('D:/pycode/code4101py/ckz.123.456').suffix
+        '.456'
+        >>> File('D:/pycode/code4101py/ckz.123..456').suffix
+        '.456'
+        >>> File('D:/pycode/.gitignore').suffix
+        ''
+        """
+        return self._path.suffix
+
+    def with_suffix(self, suffix):
+        r""" 指向同目录下后缀为suffix的文件
+
+        >>> File('a.txt').with_suffix('.py').name  # 强制替换
+        'a.py'
+        >>> File('a.txt').with_suffix('py').name  # 参考替换
+        'a.txt'
+        >>> File('a.txt').with_suffix('').name  # 删除
+        'a'
+        """
+        return File(self.abspath(self._path, suffix=suffix))
+
+    # 三、获取文件相关属性值功能
+
+    @property
+    def encoding(self):
+        """ 文件的编码
+
+        非文件、不存在时返回 None
+        """
+        if self:
+            return get_encoding(str(self))
+
+    @property
+    def size(self) -> int:
+        """ 计算文件大小
+        """
+        if self:
+            total_size = os.path.getsize(str(self))
+        else:
+            total_size = 0
+        return total_size
+
+    # 四、文件操作功能
+
+    def absdst(self, dst):
+        """ 在copy、move等中，给了个"模糊"的目标位置dst，智能推导出实际file、dir绝对路径
+        """
+        dst_ = self.abspath(dst)
+        if (isinstance(dst, str) and dst[-1] in ('\\', '/')) or dst_.is_dir():
+            dst_ = File(self.name, dst_)
+        else:
+            dst_ = File(dst_)
+        return dst_
+
+    def copy(self, dst, if_exists=None):
+        """ 复制文件
+        """
+        return self.process(dst, shutil.copy2, if_exists)
+
+    def rename(self, dst, if_exists=None):
+        r""" 文件重命名，或者也可以理解成文件移动
+        该接口和move的核心区别：move的dst是相对工作目录，而rename则是相对self.parent路径
+        """
+        # rename是move的一种特殊情况
+        return self.move(File(dst, self.parent), if_exists)
+
+    def delete(self):
+        r""" 删除自身文件
+        """
+        os.remove(str(self))
+
     # 五、其他综合性功能
 
     def read(self, *, encoding=None, mode=None):
@@ -763,9 +674,9 @@ class File:
             'b': 特殊标记，表示按二进制读取文件内容
         :return:
         """
-        if self.is_file():  # 如果存在这样的文件，那就读取文件内容
+        if self:  # 如果存在这样的文件，那就读取文件内容
             # 获得文件扩展名，并统一转成小写
-            name, suffix = self.fullpath, self.suffix
+            name, suffix = str(self), self.suffix
             if not mode: mode = suffix
             mode = mode.lower()
             if mode == 'bytes':
@@ -799,90 +710,83 @@ class File:
         else:  # 非文件对象
             raise FileNotFoundError(f'{self} 文件不存在，无法读取。')
 
-    def write(self, ob, *, encoding='utf8', if_exists='error', etag=False, mode=None):
+    def write(self, ob, *, encoding=None, if_exists=None, mode=None):
         """ 保存为文件
 
         :param ob: 写入的内容
             如果要写txt文本文件且ob不是文本对象，只会进行简单的字符串化
         :param encoding: 强制写入的编码
+            如果原文件存在且有编码，则使用原文件的编码
+            如果没有，则默认使用utf8
+            当然，其实有些格式是用不到编码信息的~~例如pkl文件
         :param if_exists: 如果文件已存在，要进行的操作
-        :param etag: 创建的文件，是否需要再进一步重命名为etag名称
         :param mode: 写入模式（例如 '.json'），默认从扩展名识别，也可以强制指定
         :return: 返回写入的文件名，这个主要是在写临时文件时有用
         """
 
-        # 1 核心功能：将ob写入文件path
-        if self.preprocess(if_exists):
-            self.ensure_dir(pathtype='file')
-            name, suffix = self.fullpath, self.suffix
+        # 将ob写入文件path
+        def get_enc():
+            # 编码在需要的时候才获取分析，减少不必要的运算开销
+            # 所以封装一个函数接口，需要的时候再计算
+            if encoding is None:
+                return self.encoding or 'utf8'
+            return encoding
+
+        if self.exist_preprcs(if_exists):
+            self.ensure_parent()
+            name, suffix = str(self), self.suffix
             if not mode: mode = suffix
             mode = mode.lower()
             if mode == '.pkl':
                 with open(name, 'wb') as f:
                     pickle.dump(ob, f)
             elif mode == '.json':
-                with open(name, 'w', encoding=encoding) as f:
+                with open(name, 'w', encoding=get_enc()) as f:
                     json.dump(ob, f, ensure_ascii=False, indent=2)
             elif mode == '.yaml':
-                with open(name, 'w', encoding=encoding) as f:
+                with open(name, 'w', encoding=get_enc()) as f:
                     yaml.dump(ob, f)
             elif isinstance(ob, bytes):
                 with open(name, 'wb') as f:
                     f.write(ob)
             else:  # 其他类型认为是文本类型
-                with open(name, 'w', errors='ignore', encoding=encoding) as f:
+                with open(name, 'w', errors='ignore', encoding=get_enc()) as f:
                     f.write(str(ob))
 
-        # 2 如果使用了etag命名机制
-        if etag:
-            return self.rename(get_etag(self.fullpath) + self.suffix, if_exists='ignore')
-        else:
-            return self
-
-    def explorer(self, proc='explorer'):
-        """ 使用windows的explorer命令打开文件
-
-        还有个类似的万能打开命令 start
-
-        :param proc: 可以自定义要执行的主程序
-        """
-        subprocess.run([proc, self.fullpath])
+        return self
 
 
 def demo_file():
     """ File类的综合测试"""
+    temp = tempfile.gettempdir()
+
     # 切换工作目录到临时文件夹
-    os.chdir(File.TEMP)
+    os.chdir(temp)
 
-    p = File('demo_path', root=File.TEMP)
+    p = File('demo_path', temp)
     p.delete()  # 如果存在先删除
-    p.ensure_dir()  # 然后再创建一个空目录
+    p.ensure_parent()  # 然后再创建一个空目录
 
-    print(File('demo_path', '.py'))
+    print(File('demo_path', suffix='.py'))
     # F:\work\CreatorTemp\demo_path.py
 
-    print(File('demo_path/', '.py'))
+    print(File('demo_path/', suffix='.py'))
     # F:\work\CreatorTemp\demo_path\tmp65m8mc0b.py
 
-    # 空字符串区别于None，会随机生成一个文件名
-    print(File('', root=File.TEMP))
+    # ...区别于None，会随机生成一个文件名
+    print(File(..., temp))
     # F:\work\CreatorTemp\tmpwp4g1692
 
     # 可以在随机名称基础上，再指定文件扩展名
-    print(File('', '.txt', root=File.TEMP))
+    print(File('', temp, suffix='.txt'))
     # F:\work\CreatorTemp\tmpimusjtu1.txt
 
 
 def demo_file_rename():
-    # 初始化一个空的测试目录
-    f = File('temp/', root=File.TEMP)
-    f.delete()
-    f.ensure_dir()
-
     # 建一个空文件
-    f1 = f / 'a.txt'
+    f1 = File('temp/a.txt', tempfile.gettempdir())
     # Path('F:/work/CreatorTemp/temp/a.txt')
-    with open(f1.fullpath, 'wb') as p: pass  # 写一个空文件
+    with open(str(f1), 'wb') as p: pass  # 写一个空文件
 
     f1.rename('A.tXt')  # 重命名
     # Path('F:/work/CreatorTemp/temp/a.txt')
@@ -924,4 +828,4 @@ class XlBytesIO(io.BytesIO):
 
 
 if __name__ == '__main__':
-    demo_path_rename()
+    demo_file_rename()
