@@ -2,7 +2,7 @@
 # -*- coding: utf-8 -*-
 # @Author : 陈坤泽
 # @Email  : 877362867@qq.com
-# @Data   : 2018/07/12 09:09
+# @Date   : 2018/07/12 09:09
 
 """
 各种文件遍历功能
@@ -10,7 +10,7 @@
 这里要强调，推荐os.walk功能
 """
 
-from pyxllib.util.xmllib import *
+from pyxllib.text import *
 import pyxllib.util.zipfile as zipfile  # 重写了标准库的zipfile文件，cp437改为gbk，解决zip中文乱码问题
 
 # 需要使用的第三方软件
@@ -73,8 +73,8 @@ class UsedRecords:
 
     def save(self):
         """保存记录文件"""
-        File(self.dirname + '/').ensure_dir()
-        File(self.fullname).write('\n'.join(self.ls), if_exists='replace')
+        File(self.dirname + '/').ensure_parent()
+        File(self.fullname).write('\n'.join(self.ls), if_exists='delete')
 
     def add(self, s):
         """新增一个使用方法
@@ -602,10 +602,10 @@ def download_file(url, fn=None, *, encoding=None, if_exists=None, ext=None, temp
     :return:
     """
     if not fn: fn = url.split('/')[-1]
-    root = File.TEMP if temp else None
-    fn = File(fn, ext, root).write(requests.get(url).content,
+    root = Dir.TEMP if temp else None
+    fn = File(fn, root, suffix=ext).write(requests.get(url).content,
                                    encoding=encoding, if_exists=if_exists, etag=(not fn))
-    return fn.fullpath
+    return fn.to_str()
 
 
 ____other = """
